#!python
"""
Extracts incidence data from tutorial data files.

Python script written for the NepidemiX tutorial of the Mathematics and HIV workshop 2014.

This script is in the public domain.
"""

import sys
from glob import glob
import numpy as np
import csv

def con_key(statevals):
    return "frozenset([('motivation', '{2}'), ('treatment', '{1}'), ('disease', '{0}')])".format(statevals[0], statevals[1], statevals[2])

def con_key_set(state_tuple):
    return set([con_key(sv) for sv in state_tuple])

def load_incidence(f_name, src, dst):
    # src, dst tuple of tuples describing the state combination in order disease, treatment, motivation.
    raw_data = []
    rowkeys = con_key_set(src)
    colkeys = con_key_set(dst)
    #    print("Src key: "+ str(rowkeys))
    #    print("Dst key: "+ str(colkeys))
    flist = glob(f_name)
    all_data = []
    print(flist)
    for ifn in flist:
        result =[]
        
        with open(ifn) as f:
            reader = csv.reader(f)
            header = reader.next()
            colnums = set([i for i in range(2,len(header)) if header[i] in colkeys])
            old_time = -1
            sum_over_time = 0
            
            for row in reader:
                if old_time != row[0]:    
                    if old_time >=0.0:
                        result.append([float(old_time), sum_over_time])
                    old_time = row[0]
                    sum_over_time = 0.0
                # Check header
                if row[1] in rowkeys:
                    # A row we want. Sum all matching columns.
                    v = 0
                    for i in range(2, len(row)):
                        if i in colnums:
                            v += float(row[i])
                    sum_over_time += v
            result.append([old_time, sum_over_time])
        all_data.append(np.array(result, dtype = float))
    return np.mean(all_data, 0), np.std(all_data, 0)


if __name__ == "__main__":
    ntFiles = glob("*nodeTrans.csv")
    if len(ntFiles) < 1:
        sys.stderr.write("No nodeTrans.csv files found. Exiting.\n")
        sys.exit(0)
    
    print("Extracting incidence data ...")
    for ntf in ntFiles:
        imean, istd = load_incidence(ntf, (('S', 'U', 'N'), ('S', 'U', 'Y')), (('A', 'U', 'N'),('A', 'U', 'Y')))
        base,ext = ntf.split('nodeTrans')
        sfname = base + 'incidence.csv'
        print("Saving to file {0}".format(sfname))
        with open(sfname, 'wb') as ofile:
            writer = csv.writer(ofile)
            for row in imean:
                writer.writerow(row)
    print("done. Exiting.")
    
